/* filter_evdev/telfilter_evdev.c
  
  Created: 2009-11
  Updated: 2009-11-27
  
  Main file of the filter_evdev program.

  This is free software licensed under the WTFPL

            DO WHAT THE FUCK YOU WANT TO PUBLIC LICENSE 
                    Version 3, November 2009 

 Copyright (C) 2009 André Gillibert
  La vieille église, 14370 Vimont, France
 Everyone is permitted to copy and distribute verbatim or modified 
 copies of this license document, and changing it is allowed as long 
 as the name is changed. 

            DO WHAT THE FUCK YOU WANT TO PUBLIC LICENSE 
   TERMS AND CONDITIONS FOR COPYING, DISTRIBUTION AND MODIFICATION 

  0. You just DO WHAT THE FUCK YOU WANT TO.
*/

/* This program is quite simple.
 * telfilter_evdev <socket> <commands>
 * e.g.
 * telfilter_evdev /tmp/evmouse_cfg "XY [0,5] *2"
 * It creates a client connection to the specified UNIX socket
 * and writes the specified commands, with an added LF terminator
 */

#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <fcntl.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

static int client_socket;
/* Creates a client connection to the unix_sockname UNIX stream server socket
 * and stores the resulting file descriptor in client_socket
 * Returns 1 on success, 0 on error
 */
static int connect_to_socket(const char *unix_sockname)
{
	struct sockaddr_un raddr={0};
	int lsock;

	raddr.sun_family = AF_UNIX;
	if (strlen(unix_sockname) >= sizeof(raddr.sun_path))
		return 0;
	strcpy(raddr.sun_path, unix_sockname);

	if ((lsock=socket(AF_UNIX, SOCK_STREAM, 0))==-1)
		return 0;
	
	if ((connect(lsock, (struct sockaddr*)&raddr, sizeof(raddr)))==-1) {
		perror("Failed to connect to socket");
		fprintf(stderr, "telfilter_evdev: Failed to connect to socket %s\n", unix_sockname);
		return 0;
	}
	client_socket = lsock;
	return 1;
}
/* Writes commands, including a LF terminator to the open client connection
 * Warning: Alters the command string.
 * Returns 1 on success, 0 on error
 */
static int send_commands(char *commands)
{
	size_t ln = strlen(commands);
	commands[ln++] = '\n'; /* overrides the nul terminator */
	return write(client_socket, commands, ln)==ln;
}
static void close_socket(void)
{
	shutdown(client_socket, SHUT_RD);
	close(client_socket);
}
int main(int argc, char **argv)
{
	int ok;
	if (argc!=3) {
		fprintf(stderr, "Usage: %s <socket> <command>\n", argv[0]);
		return 1;
	}
	if (!connect_to_socket(argv[1]))
		return 2;
	ok = send_commands(argv[2]);
	close_socket();
	return ok?0:3;
}
